@@ -1,5 +1,5 @@
 /*
- * $Id: pa_unix_oss.c,v 1.6.2.22 2005/03/08 21:26:53 aknudsen Exp $
+ * $Id: pa_unix_oss.c,v 1.6.2.28 2006/03/20 18:22:06 aknudsen Exp $
  * PortAudio Portable Real-Time Audio Library
  * Latest Version at: http://www.portaudio.com
  * OSS implementation by:
@@ -52,7 +52,10 @@
 #include <limits.h>
 #include <semaphore.h>
 
-#ifdef __linux__
+#ifdef __FreeBSD__
+# include <sys/soundcard.h>
+# define DEVICE_NAME_BASE            "/dev/dsp"
+#elif defined __linux__
 # include <linux/soundcard.h>
 # define DEVICE_NAME_BASE            "/dev/dsp"
 #else
@@ -527,12 +530,15 @@ static PaError BuildDeviceList( PaOSSHostApiRepresentation *ossApi )
            PA_UNLESS( deviceInfos = (PaDeviceInfo **) realloc( deviceInfos, maxDeviceInfos * sizeof (PaDeviceInfo *) ),
                    paInsufficientMemory );
        }
-       deviceInfos[numDevices - 1] = deviceInfo;
+       {
+           int devIdx = numDevices - 1;
+           deviceInfos[devIdx] = deviceInfo;
 
-       if( commonApi->info.defaultInputDevice == paNoDevice && deviceInfo->maxInputChannels > 0 )
-           commonApi->info.defaultInputDevice = i;
-       if( commonApi->info.defaultOutputDevice == paNoDevice && deviceInfo->maxOutputChannels > 0 )
-           commonApi->info.defaultOutputDevice = i;
+           if( commonApi->info.defaultInputDevice == paNoDevice && deviceInfo->maxInputChannels > 0 )
+               commonApi->info.defaultInputDevice = devIdx;
+           if( commonApi->info.defaultOutputDevice == paNoDevice && deviceInfo->maxOutputChannels > 0 )
+               commonApi->info.defaultOutputDevice = devIdx;
+       }
     }
 
     /* Make an array of PaDeviceInfo pointers out of the linked list */
@@ -1061,6 +1067,7 @@ static PaError PaOssStreamComponent_Read( PaOssStreamComponent *component, unsig
 
     ENSURE_( bytesRead = read( component->fd, component->buffer, len ), paUnanticipatedHostError );
     *frames = bytesRead / PaOssStreamComponent_FrameSize( component );
+    /* TODO: Handle condition where number of frames read doesn't equal number of frames requested */
 
 error:
     return result;
@@ -1074,6 +1081,7 @@ static PaError PaOssStreamComponent_Write( PaOssStreamComponent *component, unsi
 
     ENSURE_( bytesWritten = write( component->fd, component->buffer, len ), paUnanticipatedHostError );
     *frames = bytesWritten / PaOssStreamComponent_FrameSize( component );
+    /* TODO: Handle condition where number of frames written doesn't equal number of frames requested */
 
 error:
     return result;
@@ -1127,7 +1135,7 @@ static PaError PaOssStream_Configure( PaOssStream *stream, double sampleRate, un
     stream->framesPerHostBuffer = framesPerHostBuffer;
     stream->pollTimeout = (int) ceil( 1e6 * framesPerHostBuffer / sampleRate );    /* Period in usecs, rounded up */
 
-    stream->streamRepresentation.streamInfo.sampleRate = sampleRate;
+    stream->sampleRate = stream->streamRepresentation.streamInfo.sampleRate = sampleRate;
 
 error:
     return result;
@@ -1204,7 +1212,7 @@ static PaError OpenStream( struct PaUtilHostApiRepresentation *hostApi,
     
     /* allocate and do basic initialization of the stream structure */
     PA_UNLESS( stream = (PaOssStream*)PaUtil_AllocateMemory( sizeof(PaOssStream) ), paInsufficientMemory );
-    PaOssStream_Initialize( stream, inputParameters, outputParameters, streamCallback, userData, streamFlags, ossHostApi );
+    PA_ENSURE( PaOssStream_Initialize( stream, inputParameters, outputParameters, streamCallback, userData, streamFlags, ossHostApi ) );
 
     PA_ENSURE( PaOssStream_Configure( stream, sampleRate, framesPerBuffer, &inLatency, &outLatency ) );
 
@@ -1528,6 +1536,7 @@ static void *PaOSS_AudioThreadProc( void *userData )
     int triggered = stream->triggered;  /* See if SNDCTL_DSP_TRIGGER has been issued already */
     int initiateProcessing = triggered;    /* Already triggered? */
     PaStreamCallbackFlags cbFlags = 0;  /* We might want to keep state across iterations */
+    PaStreamCallbackTimeInfo timeInfo = {0,0,0}; /* TODO: IMPLEMENT ME */
     
     /*
 #if ( SOUND_VERSION > 0x030904 )
@@ -1557,8 +1566,6 @@ static void *PaOSS_AudioThreadProc( void *userData )
 
     while( 1 )
     {
-        PaStreamCallbackTimeInfo timeInfo = {0,0,0}; /* TODO: IMPLEMENT ME */
-
         pthread_testcancel();
 
         if( stream->callbackStop && callbackResult == paContinue )
@@ -1787,14 +1794,14 @@ static PaTime GetStreamTime( PaStream *s )
 
     if( stream->playback ) {
         if( ioctl( stream->playback->fd, SNDCTL_DSP_GETOPTR, &info) == 0 ) {
-            delta = ( info.bytes - stream->lastPosPtr ) & 0x000FFFFF;
-            return ( stream->lastStreamBytes + delta) / PaOssStreamComponent_FrameSize( stream->playback ) / stream->sampleRate;
+            delta = ( info.bytes - stream->lastPosPtr ) /* & 0x000FFFFF*/;
+            return (float)(stream->lastStreamBytes + delta) / PaOssStreamComponent_FrameSize( stream->playback ) / stream->sampleRate;
         }
     }
     else {
         if (ioctl( stream->capture->fd, SNDCTL_DSP_GETIPTR, &info) == 0) {
-            delta = (info.bytes - stream->lastPosPtr) & 0x000FFFFF;
-            return ( stream->lastStreamBytes + delta) / PaOssStreamComponent_FrameSize( stream->capture ) / stream->sampleRate;
+            delta = (info.bytes - stream->lastPosPtr) /*& 0x000FFFFF*/;
+            return (float)(stream->lastStreamBytes + delta) / PaOssStreamComponent_FrameSize( stream->capture ) / stream->sampleRate;
         }
     }
 
@@ -1856,9 +1863,7 @@ static PaError ReadStream( PaStream* s,
 }
 
 
-static PaError WriteStream( PaStream* s,
-                            const void *buffer,
-                            unsigned long frames )
+static PaError WriteStream( PaStream *s, const void *buffer, unsigned long frames )
 {
     PaOssStream *stream = (PaOssStream*)s;
     int bytesRequested, bytesWritten;
@@ -1869,8 +1874,9 @@ static PaError WriteStream( PaStream* s,
      * so we copy the user provided pointers */
     if( stream->bufferProcessor.userOutputIsInterleaved )
         userBuffer = buffer;
-    else /* Copy channels into local array */
+    else
     {
+        /* Copy channels into local array */
         userBuffer = stream->playback->userBuffers;
         memcpy( (void *)userBuffer, buffer, sizeof (void *) * stream->playback->userChannelCount );
     }
