@@ -26,25 +26,20 @@
 #define int32 long
 #endif
 
-#if defined(__FreeBSD__) || defined(MACOSX)
+#if defined(__FreeBSD__) || defined(__APPLE__)
 #include <machine/endian.h>
 #endif
 
-#ifdef MACOSX
-#define __BYTE_ORDER BYTE_ORDER
-#define __LITTLE_ENDIAN LITTLE_ENDIAN
-#endif                                                                          
-
 #ifdef __linux__
 #include <endian.h>
 #endif
 
-#if defined(__unix__) || defined(MACOSX)
-#if !defined(__BYTE_ORDER) || !defined(__LITTLE_ENDIAN)                         
+#if defined(__unix__) || defined(__APPLE__)
+#if !defined(BYTE_ORDER) || !defined(LITTLE_ENDIAN)                         
 #error No byte order defined                                                    
 #endif                                                                          
 
-#if __BYTE_ORDER == __LITTLE_ENDIAN                                             
+#if BYTE_ORDER == LITTLE_ENDIAN                                             
 #define HIOFFSET 1                                                              
 #define LOWOFFSET 0                                                             
 #else                                                                           
@@ -53,8 +48,7 @@
 #endif /* __BYTE_ORDER */                                                       
 #include <sys/types.h>
 #define int32 int32_t
-#endif /* __unix__ or MACOSX*/
-
+#endif /* __unix__ or __APPLE__*/
 
 union tabfudge
 {
@@ -62,7 +56,6 @@ union tabfudge
     int32 tf_i[2];
 };
 
-
 /* -------------------------- phasor~ ------------------------------ */
 static t_class *phasor_class, *scalarphasor_class;
 
