@@ -785,10 +785,13 @@ static t_gobj *canvas_findhitbox(t_canvas *x, int xpos, int ypos,
     int *x1p, int *y1p, int *x2p, int *y2p)
 {
     t_gobj *y, *rval = 0;
+    int x1, y1, x2, y2;
+    *x1p = -0x7fffffff;
     for (y = x->gl_list; y; y = y->g_next)
     {
-        if (canvas_hitbox(x, y, xpos, ypos, x1p, y1p, x2p, y2p))
-            rval = y; 
+        if (canvas_hitbox(x, y, xpos, ypos, &x1, &y1, &x2, &y2)
+            && (x1 > *x1p))
+                *x1p = x1, *y1p = y1, *x2p = x2, *y2p = y2, rval = y; 
     }
     return (rval);
 }
@@ -818,7 +821,7 @@ void canvas_setgraph(t_glist *x, int flag, int nogoprect)
             canvas_fixlinesfor(x->gl_owner, &x->gl_obj);
         }
     }
-    else if (flag && !glist_isgraph(x))
+    else if (flag)
     {
         if (x->gl_pixwidth <= 0)
             x->gl_pixwidth = GLIST_DEFGRAPHWIDTH;
@@ -829,6 +832,7 @@ void canvas_setgraph(t_glist *x, int flag, int nogoprect)
         if (x->gl_owner && !x->gl_loading && glist_isvisible(x->gl_owner))
             gobj_vis(&x->gl_gobj, x->gl_owner, 0);
         x->gl_isgraph = 1;
+        x->gl_hidetext = !(!(flag&2));
         if (!nogoprect && !x->gl_goprect)
         {
             t_gobj *g;
@@ -864,7 +868,7 @@ void canvas_properties(t_glist *x)
         sprintf(graphbuf,
             "pdtk_canvas_dialog %%s %g %g %d %g %g %g %g %d %d %d %d\n",
                 0., 0.,
-                1,
+                glist_isgraph(x) ,//1,
                 x->gl_x1, x->gl_y1, x->gl_x2, x->gl_y2, 
                 (int)x->gl_pixwidth, (int)x->gl_pixheight,
                 (int)x->gl_xmargin, (int)x->gl_ymargin);
@@ -894,7 +898,7 @@ static void canvas_donecanvasdialog(t_glist *x,
 
     xperpix = atom_getfloatarg(0, argc, argv);
     yperpix = atom_getfloatarg(1, argc, argv);
-    graphme = (atom_getfloatarg(2, argc, argv) != 0);
+    graphme = (int)(atom_getfloatarg(2, argc, argv));
     x1 = atom_getfloatarg(3, argc, argv);
     y1 = atom_getfloatarg(4, argc, argv);
     x2 = atom_getfloatarg(5, argc, argv);
@@ -1014,11 +1018,7 @@ static void canvas_done_popup(t_canvas *x, float which, float xpos, float ypos)
     if (which == 0)
         canvas_properties(x);
     else if (which == 2)
-    {
-        strcpy(pathbuf, sys_libdir->s_name);
-        strcat(pathbuf, "/doc/5.reference/0.INTRO.txt");
-        sys_vgui("menu_opentext %s\n", pathbuf);
-    }
+        open_via_helppath("intro.pd", canvas_getdir((t_canvas *)x)->s_name);
 }
 
 #define NOMOD 0
@@ -1027,12 +1027,6 @@ static void canvas_done_popup(t_canvas *x, float which, float xpos, float ypos)
 #define ALTMOD 4
 #define RIGHTCLICK 8
 
-/* on one-button-mouse machines, you can use double click to
-    mean right click (which gets the popup menu.)  Do this for Mac. */
-#ifdef MACOSX
-#define SIMULATERIGHTCLICK
-#endif
-
 static double canvas_upclicktime;
 static int canvas_upx, canvas_upy;
 #define DCLICKINTERVAL 0.25
@@ -1067,11 +1061,7 @@ void canvas_doclick(t_canvas *x, int xpos, int ypos, int which,
 
     if (doit && !runmode && xpos == canvas_upx && ypos == canvas_upy &&
         sys_getrealtime() - canvas_upclicktime < DCLICKINTERVAL)
-#ifdef SIMULATERIGHTCLICK
-            rightclick = 1;
-#else
             doublemod = 1;
-#endif
     x->gl_editor->e_lastmoved = 0;
     if (doit)
     {
@@ -1476,7 +1466,7 @@ void canvas_key(t_canvas *x, t_symbol *s, int ac, t_atom *av)
     keynum = (av[1].a_type == A_FLOAT ? av[1].a_w.w_float : 0);
     if (keynum == '\\' || keynum == '{' || keynum == '}')
     {
-        post("%c: dropped", (int)keynum);
+        post("keycode %d: dropped", (int)keynum);
         return;
     }
 #if 0
@@ -1492,7 +1482,7 @@ void canvas_key(t_canvas *x, t_symbol *s, int ac, t_atom *av)
         keyupsym = gensym("#keyup");
         keynamesym = gensym("#keyname");
     }
-#ifdef MACOSX
+#ifdef __APPLE__
         if (keynum == 30)
             keynum = 0, gotkeysym = gensym("Up");
         else if (keynum == 31)
@@ -1957,8 +1947,12 @@ static void canvas_copy(t_canvas *x)
         int bufsize;
         rtext_getseltext(x->gl_editor->e_textedfor, &buf, &bufsize);
 
-#ifdef UNIX
-            /* in X windows (i.e., UNIX) the selection already went to the
+#if defined(MSW) || defined(__APPLE__)
+            /* for Mac or Windows, copy the text to the clipboard here */
+        sys_vgui("clipboard clear\n", bufsize, buf);
+        sys_vgui("clipboard append {%.*s}\n", bufsize, buf);
+#else
+            /* in X windows the selection already went to the
             clipboard when it was made; here we "copy" it to our own buffer
             as well, because, annoyingly, the clipboard will usually be 
             destroyed by the time the user asks to "paste". */
@@ -1967,11 +1961,7 @@ static void canvas_copy(t_canvas *x)
         canvas_textcopybuf = (char *)getbytes(bufsize);
         memcpy(canvas_textcopybuf, buf, bufsize);
         canvas_textcopybufsize = bufsize;
-#else /* UNIX */
-            /* otherwise just copy the text to the clipboard here */
-        sys_vgui("clipboard clear\n", bufsize, buf);
-        sys_vgui("clipboard append {%.*s}\n", bufsize, buf);
-#endif /* UNIX */
+#endif
     }
 }
 
@@ -2121,19 +2111,18 @@ static void canvas_paste(t_canvas *x)
     if (x->gl_editor->e_textedfor)
     {
             /* simulate keystrokes as if the copy buffer were typed in. */
-#ifdef UNIX
-            /* in UNIX we kept the text in our own copy buffer */
+#if defined(MSW) || defined(__APPLE__)
+            /* for Mac or Windows,  ask the GUI to send the clipboard down */
+        sys_gui("pdtk_pastetext\n");
+#else
+            /* in X windows we kept the text in our own copy buffer */
         int i;
         for (i = 0; i < canvas_textcopybufsize; i++)
         {
             pd_vmess(&x->gl_gobj.g_pd, gensym("key"), "iii",
                 1, canvas_textcopybuf[i]&0xff, 0);
         }
-#else /* UNIX */
-            /* otherwise appeal to the GUI to send the clipboard down */
-        sys_gui("pdtk_pastetext\n");
-#endif /* UNIX */
-
+#endif
     }
     else
     {
