@@ -2,10 +2,13 @@
 * For information on usage and redistribution, and for a DISCLAIMER OF ALL
 * WARRANTIES, see the file, "LICENSE.txt," in this distribution.  */
 
-/* this file reads and writes the "data" portions of a canvas to a file.
-See also canvas_saveto(), etc., in g_editor.c.  The data portion is a
-collection of "scalar" objects.  Routines here can save collections of
-scalars into a file and reload them; also, support is included here for
+/* 
+Routines to read and write canvases to files:
+canvas_savetofile() writes a root canvas to a "pd" file.  (Reading "pd" files
+is done simply by passing the contents to the pd message interpreter.)
+Alternatively, the  glist_read() and glist_write() routines read and write
+"data" from and to files (reading reads into an existing canvas), using a
+file format as in the dialog window for data.
 */
 
 #include <stdlib.h>
@@ -14,6 +17,9 @@ scalars into a file and reload them; also, support is included here for
 #include "g_canvas.h"
 #include <string.h>
 
+static t_class *declare_class;
+void canvas_savedeclarationsto(t_canvas *x, t_binbuf *b);
+
     /* the following routines read "scalars" from a file into a canvas. */
 
 static int canvas_scanbinbuf(int natoms, t_atom *vec, int *p_indexout,
@@ -566,22 +572,31 @@ static void canvas_saveto(t_canvas *x, t_binbuf *b)
         /* subpatch */
     if (x->gl_owner && !x->gl_env)
     {
+        /* have to go to original binbuf to find out how we were named. */
+        t_binbuf *bz = binbuf_new();
+        t_symbol *patchsym;
+        binbuf_addbinbuf(bz, x->gl_obj.ob_binbuf);
+        patchsym = atom_getsymbolarg(1, binbuf_getnatom(bz), binbuf_getvec(bz));
+        binbuf_free(bz);
         binbuf_addv(b, "ssiiiisi;", gensym("#N"), gensym("canvas"),
             (int)(x->gl_screenx1),
             (int)(x->gl_screeny1),
             (int)(x->gl_screenx2 - x->gl_screenx1),
             (int)(x->gl_screeny2 - x->gl_screeny1),
-            (*x->gl_name->s_name ? x->gl_name: gensym("(subpatch)")),
+            (patchsym != &s_ ? patchsym: gensym("(subpatch)")),
             x->gl_mapped);
     }
         /* root or abstraction */
-    else binbuf_addv(b, "ssiiiii;", gensym("#N"), gensym("canvas"),
+    else 
+    {
+        binbuf_addv(b, "ssiiiii;", gensym("#N"), gensym("canvas"),
             (int)(x->gl_screenx1),
             (int)(x->gl_screeny1),
             (int)(x->gl_screenx2 - x->gl_screenx1),
             (int)(x->gl_screeny2 - x->gl_screeny1),
                 (int)x->gl_font);
-
+        canvas_savedeclarationsto(x, b);
+    }
     for (y = x->gl_list; y; y = y->g_next)
         gobj_save(y, b);
 
@@ -606,7 +621,8 @@ static void canvas_saveto(t_canvas *x, t_binbuf *b)
                 x->gl_x1, x->gl_y1,
                 x->gl_x2, x->gl_y2,
                 (float)x->gl_pixwidth, (float)x->gl_pixheight,
-                1., (float)x->gl_xmargin, (float)x->gl_ymargin); 
+                (float)((x->gl_hidetext)?2.:1.),
+                (float)x->gl_xmargin, (float)x->gl_ymargin); 
                     /* otherwise write in 0.38-compatible form */
         else binbuf_addv(b, "ssfffffff;", gensym("#X"), gensym("coords"),
                 x->gl_x1, x->gl_y1,
@@ -714,7 +730,6 @@ static void canvas_menusave(t_canvas *x)
     else canvas_menusaveas(x2);
 }
 
-
 void g_readwrite_setup(void)
 {
     class_addmethod(canvas_class, (t_method)glist_write,
