@@ -30,8 +30,8 @@ t_widgetbehavior text_widgetbehavior;
 
 /* ----------------- the "text" object.  ------------------ */
 
-    /* add a "text" object (comment) to a glist.  While this one goes for any glist,
-    the other 3 below are for canvases only.  (why?)  This is called
+    /* add a "text" object (comment) to a glist.  While this one goes for any
+    glist, the other 3 below are for canvases only.  (why?)  This is called
     without args if invoked from the GUI; otherwise at least x and y
     are provided.  */
 
@@ -359,7 +359,11 @@ static void message_adddollar(t_message *x, t_floatarg f)
 static void message_adddollsym(t_message *x, t_symbol *s)
 {
     t_atom a;
-    SETDOLLSYM(&a, s);
+    char buf[MAXPDSTRING];
+    buf[0] = '$';
+    strncpy(buf+1, s->s_name, MAXPDSTRING-2);
+    buf[MAXPDSTRING-1] = 0;
+    SETDOLLSYM(&a, gensym(buf));
     binbuf_add(x->m_text.te_binbuf, 1, &a);
     glist_retext(x->m_glist, &x->m_text);
 }
@@ -566,6 +570,19 @@ static void gatom_symbol(t_gatom *x, t_symbol *s)
     gatom_bang(x);
 }
 
+    /* We need a list method because, since there's both an "inlet" and a
+    "nofirstin" flag, the standard list behavior gets confused. */
+static void gatom_list(t_gatom *x, t_symbol *s, int argc, t_atom *argv)
+{
+    if (!argc)
+        gatom_bang(x);
+    else if (argv->a_type == A_FLOAT)
+        gatom_float(x, argv->a_w.w_float);
+    else if (argv->a_type == A_SYMBOL)
+        gatom_symbol(x, argv->a_w.w_symbol);
+    else pd_error(x, "gatom_list: need float or symbol");
+}
+
 static void gatom_motion(void *z, t_floatarg dx, t_floatarg dy)
 {
     t_gatom *x = (t_gatom *)z;
@@ -889,7 +906,7 @@ static void gatom_properties(t_gobj *z, t_glist *owner)
 {
     t_gatom *x = (t_gatom *)z;
     char buf[200];
-    sprintf(buf, "pdtk_gatom_dialog %%s %d %g %g %d %s %s %s\n",
+    sprintf(buf, "pdtk_gatom_dialog %%s %d %g %g %d {%s} {%s} {%s}\n",
         x->a_text.te_width, x->a_draglo, x->a_draghi,
             x->a_wherelabel, gatom_escapit(x->a_label)->s_name,
                 gatom_escapit(x->a_symfrom)->s_name,
@@ -1133,7 +1150,7 @@ static t_widgetbehavior gatom_widgetbehavior =
 
 /* -------------------- the "text" class  ------------ */
 
-#ifdef MACOSX
+#ifdef __APPLE__
 #define EXTRAPIX 2
 #else
 #define EXTRAPIX 1
@@ -1349,6 +1366,7 @@ void g_text_setup(void)
     class_addbang(gatom_class, gatom_bang);
     class_addfloat(gatom_class, gatom_float);
     class_addsymbol(gatom_class, gatom_symbol);
+    class_addlist(gatom_class, gatom_list);
     class_addmethod(gatom_class, (t_method)gatom_set, gensym("set"),
         A_GIMME, 0);
     class_addmethod(gatom_class, (t_method)gatom_click, gensym("click"),
