@@ -131,9 +131,19 @@ static void audio_init( void)
 void sys_setchsr(int chin, int chout, int sr)
 {
     int nblk;
-    int inbytes = (chin ? chin : 2) * (DEFDACBLKSIZE*sizeof(float));
-    int outbytes = (chout ? chout : 2) * (DEFDACBLKSIZE*sizeof(float));
+    int inbytes = (chin ? chin : 2) *
+                (DEFDACBLKSIZE*sizeof(float));
+    int outbytes = (chout ? chout : 2) *
+                (DEFDACBLKSIZE*sizeof(float));
 
+    if (sys_soundin)
+        freebytes(sys_soundin, 
+            (sys_inchannels? sys_inchannels : 2) *
+                (DEFDACBLKSIZE*sizeof(float)));
+    if (sys_soundout)
+        freebytes(sys_soundout, 
+            (sys_outchannels? sys_outchannels : 2) *
+                (DEFDACBLKSIZE*sizeof(float)));
     sys_inchannels = chin;
     sys_outchannels = chout;
     sys_dacsr = sr;
@@ -141,14 +151,10 @@ void sys_setchsr(int chin, int chout, int sr)
     if (sys_advance_samples < 3 * DEFDACBLKSIZE)
         sys_advance_samples = 3 * DEFDACBLKSIZE;
 
-    if (sys_soundin)
-        free(sys_soundin);
-    sys_soundin = (t_float *)malloc(inbytes);
+    sys_soundin = (t_float *)getbytes(inbytes);
     memset(sys_soundin, 0, inbytes);
 
-    if (sys_soundout)
-        free(sys_soundout);
-    sys_soundout = (t_float *)malloc(outbytes);
+    sys_soundout = (t_float *)getbytes(outbytes);
     memset(sys_soundout, 0, outbytes);
 
     if (sys_verbose)
@@ -170,7 +176,8 @@ void sys_open_audio(int naudioindev, int *audioindev, int nchindev,
 {
     int i, *ip;
     int defaultchannels = SYS_DEFAULTCH;
-    int inchans, outchans;
+    int inchans, outchans, nrealindev, nrealoutdev;
+    int realindev[MAXAUDIOINDEV], realoutdev[MAXAUDIOOUTDEV];
     int realinchans[MAXAUDIOINDEV], realoutchans[MAXAUDIOOUTDEV];
 
     char indevlist[MAXNDEV*DEVDESCSIZE], outdevlist[MAXNDEV*DEVDESCSIZE];
@@ -296,10 +303,22 @@ void sys_open_audio(int naudioindev, int *audioindev, int nchindev,
     }
     
         /* count total number of input and output channels */
-    for (i = inchans = 0; i < naudioindev; i++)
-        inchans += (realinchans[i] = (chindev[i] > 0 ? chindev[i] : 0));
-    for (i = outchans = 0; i < naudiooutdev; i++)
-        outchans += (realoutchans[i] = (choutdev[i] > 0 ? choutdev[i] : 0));
+    for (i = nrealindev = inchans = 0; i < naudioindev; i++)
+        if (chindev[i] > 0)
+    {
+        realinchans[nrealindev] = chindev[i];
+        realindev[nrealindev] = audioindev[i];
+        inchans += chindev[i];
+        nrealindev++;
+    }
+    for (i = nrealoutdev = outchans = 0; i < naudiooutdev; i++)
+        if (choutdev[i] > 0)
+    {
+        realoutchans[nrealoutdev] = choutdev[i];
+        realoutdev[nrealoutdev] = audiooutdev[i];
+        outchans += choutdev[i];
+        nrealoutdev++;
+    }
         /* if no input or output devices seem to have been specified,
         this really means just disable audio, which we now do. */
     if (!inchans && !outchans)
@@ -322,23 +341,23 @@ else
 #endif
 #ifdef USEAPI_JACK
         if (sys_audioapi == API_JACK) 
-            jack_open_audio((naudioindev > 0 ? realinchans[0] : 0),
-                            (naudiooutdev > 0 ? realoutchans[0] : 0), rate);
+            jack_open_audio((nrealindev > 0 ? realinchans[0] : 0),
+                (nrealoutdev > 0 ? realoutchans[0] : 0), rate);
 
         else
 #endif    
 #ifdef USEAPI_OSS
         if (sys_audioapi == API_OSS)
-            oss_open_audio(naudioindev, audioindev, nchindev, realinchans,
-                    naudiooutdev, audiooutdev, nchoutdev, realoutchans, rate);
+            oss_open_audio(nrealindev, realindev, nrealindev, realinchans,
+                nrealoutdev, realoutdev, nrealoutdev, realoutchans, rate);
         else
 #endif
 #ifdef USEAPI_ALSA
             /* for alsa, only one device is supported; it may
             be open for both input and output. */
         if (sys_audioapi == API_ALSA)
-            alsa_open_audio(naudioindev, audioindev, nchindev, realinchans,
-                    naudiooutdev, audiooutdev, nchoutdev, realoutchans, rate);
+            alsa_open_audio(nrealindev, audioindev, nrealindev, realinchans,
+                nrealoutdev, audiooutdev, nrealoutdev, realoutchans, rate);
         else 
 #endif
 #ifdef USEAPI_SGI
@@ -354,8 +373,8 @@ else
 #endif
 #ifdef USEAPI_MMIO
         if (sys_audioapi == API_MMIO)
-            mmio_open_audio(naudioindev, audioindev, nchindev, realinchans,
-                    naudiooutdev, audiooutdev, nchoutdev, realoutchans, rate);
+            mmio_open_audio(nrealindev, audioindev, nrealindev, realinchans,
+                nrealoutdev, audiooutdev, nrealoutdev, realoutchans, rate);
         else
 #endif
             post("unknown audio API specified");
@@ -588,11 +607,6 @@ static void audio_getdevs(char *indevlist, int *nindevs,
     }
 }
 
-#ifdef MSW
-#define DEVONSET 0  /* microsoft device list starts at 0 (the "mapper"). */
-#else               /* (see also MSW ifdef in sys_parsedevlist(), s_main.c)  */
-#define DEVONSET 1  /* To agree with command line flags, normally start at 1 */
-#endif
 
 static void sys_listaudiodevs(void )
 {
@@ -606,17 +620,23 @@ static void sys_listaudiodevs(void )
         post("no audio input devices found");
     else
     {
-        post("input devices:");
+            /* To agree with command line flags, normally start at 1 */
+            /* But microsoft "MMIO" device list starts at 0 (the "mapper"). */
+            /* (see also sys_mmio variable in s_main.c)  */
+
+        post("audio input devices:");
         for (i = 0; i < nindevs; i++)
-            post("%d. %s", i+1, indevlist + i * DEVDESCSIZE);
+            post("%d. %s", i + (sys_audioapi != API_MMIO),
+                indevlist + i * DEVDESCSIZE);
     }
     if (!noutdevs)
         post("no audio output devices found");
     else
     {
-        post("output devices:");
+        post("audio output devices:");
         for (i = 0; i < noutdevs; i++)
-            post("%d. %s", i + DEVONSET, outdevlist + i * DEVDESCSIZE);
+            post("%d. %s", i + (sys_audioapi != API_MMIO),
+                outdevlist + i * DEVDESCSIZE);
     }
     post("API number %d\n", sys_audioapi);
 }
@@ -638,29 +658,18 @@ void glob_audio_properties(t_pd *dummy, t_floatarg flongform)
     char indevlist[MAXNDEV*DEVDESCSIZE], outdevlist[MAXNDEV*DEVDESCSIZE];
     int nindevs = 0, noutdevs = 0, canmulti = 0, i;
 
-    char indevliststring[MAXNDEV*(DEVDESCSIZE+4)+80],
-        outdevliststring[MAXNDEV*(DEVDESCSIZE+4)+80];
-
     audio_getdevs(indevlist, &nindevs, outdevlist, &noutdevs, &canmulti,
         MAXNDEV, DEVDESCSIZE);
 
-    strcpy(indevliststring, "{");
+    sys_gui("global audio_indevlist; set audio_indevlist {}\n");
     for (i = 0; i < nindevs; i++)
-    {
-        strcat(indevliststring, "\"");
-        strcat(indevliststring, indevlist + i * DEVDESCSIZE);
-        strcat(indevliststring, "\" ");
-    }
-    strcat(indevliststring, "}");
+        sys_vgui("lappend audio_indevlist \"%s\"\n",
+            indevlist + i * DEVDESCSIZE);
 
-    strcpy(outdevliststring, "{");
+    sys_gui("global audio_outdevlist; set audio_outdevlist {}\n");
     for (i = 0; i < noutdevs; i++)
-    {
-        strcat(outdevliststring, "\"");
-        strcat(outdevliststring, outdevlist + i * DEVDESCSIZE);
-        strcat(outdevliststring, "\" ");
-    }
-    strcat(outdevliststring, "}");
+        sys_vgui("lappend audio_outdevlist \"%s\"\n",
+            outdevlist + i * DEVDESCSIZE);
 
     sys_get_audio_params(&naudioindev, audioindev, chindev,
         &naudiooutdev, audiooutdev, choutdev, &rate, &advance);
@@ -670,7 +679,6 @@ void glob_audio_properties(t_pd *dummy, t_floatarg flongform)
     if (naudioindev > 1 || naudiooutdev > 1)
         flongform = 1;
 
-
     audioindev1 = (naudioindev > 0 &&  audioindev[0]>= 0 ? audioindev[0] : 0);
     audioindev2 = (naudioindev > 1 &&  audioindev[1]>= 0 ? audioindev[1] : 0);
     audioindev3 = (naudioindev > 2 &&  audioindev[2]>= 0 ? audioindev[2] : 0);
@@ -689,13 +697,11 @@ void glob_audio_properties(t_pd *dummy, t_floatarg flongform)
     audiooutchan4 = (naudiooutdev > 3 ? choutdev[3] : 0);
     sprintf(buf,
 "pdtk_audio_dialog %%s \
-%s %d %d %d %d %d %d %d %d \
-%s %d %d %d %d %d %d %d %d \
+%d %d %d %d %d %d %d %d \
+%d %d %d %d %d %d %d %d \
 %d %d %d %d\n",
-        indevliststring,
         audioindev1, audioindev2, audioindev3, audioindev4, 
         audioinchan1, audioinchan2, audioinchan3, audioinchan4, 
-        outdevliststring,
         audiooutdev1, audiooutdev2, audiooutdev3, audiooutdev4,
         audiooutchan1, audiooutchan2, audiooutchan3, audiooutchan4, 
         rate, advance, canmulti, (flongform != 0));
