@@ -40,7 +40,7 @@ typedef int socklen_t;
 #include <string.h>
 #include <stdio.h>
 
-#ifdef MACOSX
+#ifdef __APPLE__
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <pthread.h>
@@ -85,7 +85,7 @@ struct _socketreceiver
     t_socketreceivefn sr_socketreceivefn;
 };
 
-extern char pd_version[];
+extern char *pd_version;
 extern int sys_guisetportnumber;
 extern char sys_font[]; /* tb: typeface */
 
@@ -788,6 +788,8 @@ void sys_queuegui(void *client, t_glist *glist, t_guicallbackfn f)
         for (gq = sys_guiqueuehead; gq->gq_next; gq = gq->gq_next)
             if (gq->gq_client == client)
                 return;
+        if (gq->gq_client == client)
+            return;
         gqnextptr = &gq->gq_next;
     }
     gq = t_getbytes(sizeof(*gq));
@@ -845,6 +847,7 @@ void glob_watchdog(t_pd *dummy)
 static int defaultfontshit[] = {
     8, 5, 9, 10, 6, 10, 12, 7, 13, 14, 9, 17, 16, 10, 19, 24, 15, 28,
         24, 15, 28};
+#define NDEFAULTFONT (sizeof(defaultfontshit)/sizeof(*defaultfontshit))
 
 int sys_startgui(const char *guidir)
 {
@@ -892,7 +895,7 @@ int sys_startgui(const char *guidir)
     {
             /* fake the GUI's message giving cwd and font sizes; then
             skip starting the GUI up. */
-        t_atom zz[19];
+        t_atom zz[NDEFAULTFONT+2];
         int i;
 #ifdef MSW
         if (GetCurrentDirectory(MAXPDSTRING, cmdbuf) == 0)
@@ -904,9 +907,9 @@ int sys_startgui(const char *guidir)
         
 #endif
         SETSYMBOL(zz, gensym(cmdbuf));
-        for (i = 1; i < 22; i++)
-            SETFLOAT(zz + i, defaultfontshit[i-1]);
-        SETFLOAT(zz+22,0);
+        for (i = 0; i < (int)NDEFAULTFONT; i++)
+            SETFLOAT(zz+i+1, defaultfontshit[i]);
+        SETFLOAT(zz+NDEFAULTFONT+1,0);
         glob_initfromgui(0, 0, 23, zz);
     }
     else if (sys_guisetportnumber)  /* GUI exists and sent us a port number */
@@ -1018,7 +1021,7 @@ int sys_startgui(const char *guidir)
         else if (!childpid)                     /* we're the child */
         {
             seteuid(getuid());          /* lose setuid priveliges */
-#ifndef MACOSX
+#ifndef __APPLE__
                 /* the wish process in Unix will make a wish shell and
                     read/write standard in and out unless we close the
                     file descriptors.  Somehow this doesn't make the MAC OSX
@@ -1037,7 +1040,7 @@ int sys_startgui(const char *guidir)
 #endif
             if (!sys_guicmd)
             {
-#ifdef MACOSX
+#ifdef __APPLE__
                 char *homedir = getenv("HOME"), filename[250];
                 struct stat statbuf;
                     /* first look for Wish bundled with and renamed "Pd" */
@@ -1202,7 +1205,7 @@ int sys_startgui(const char *guidir)
     if (!SetPriorityClass(GetCurrentProcess(), HIGH_PRIORITY_CLASS))
         fprintf(stderr, "pd: couldn't set high priority class\n");
 #endif
-#ifdef MACOSX
+#ifdef __APPLE__
     if (sys_hipriority)
     {
         struct sched_param param;
@@ -1214,7 +1217,7 @@ int sys_startgui(const char *guidir)
         if (err)
             post("warning: high priority scheduling failed\n");
     }
-#endif /* MACOSX */
+#endif /* __APPLE__ */
 
     if (!sys_nogui && !sys_guisetportnumber)
     {
