@@ -2,10 +2,6 @@
 * For information on usage and redistribution, and for a DISCLAIMER OF ALL
 * WARRANTIES, see the file, "LICENSE.txt," in this distribution.  */
 
-char pd_version[] = "Pd version 0.39-2\n";
-char pd_compiletime[] = __TIME__;
-char pd_compiledate[] = __DATE__;
-
 #include "m_pd.h"
 #include "m_imp.h"
 #include "s_stuff.h"
@@ -26,6 +22,10 @@ char pd_compiledate[] = __DATE__;
 #include <winbase.h>
 #endif
 
+char *pd_version;
+char pd_compiletime[] = __TIME__;
+char pd_compiledate[] = __DATE__;
+
 void pd_init(void);
 int sys_argparse(int argc, char **argv);
 void sys_findprogdir(char *progname);
@@ -33,7 +33,9 @@ int sys_startgui(const char *guipath);
 int sys_rcfile(void);
 int m_scheduler(void);
 void sys_addhelppath(char *p);
+#ifdef USEAPI_ALSA
 void alsa_adddev(char *name);
+#endif
 
 int sys_debuglevel;
 int sys_verbose;
@@ -56,9 +58,10 @@ int sys_nmidiin = -1;
 int sys_midiindevlist[MAXMIDIINDEV] = {1};
 int sys_midioutdevlist[MAXMIDIOUTDEV] = {1};
 
-char sys_font[] = "courier"; /* tb: font name */
+char sys_font[100] = "courier"; /* tb: font name */
 static int sys_main_srate;
 static int sys_main_advance;
+static int sys_listplease;
 
 int sys_externalschedlib;
 char sys_externalschedlibname[MAXPDSTRING];
@@ -103,7 +106,7 @@ typedef struct _fontinfo
     in the six fonts.  */
 
 static t_fontinfo sys_fontlist[] = {
-    {8, 5, 9, 0, 0, 0}, {10, 7, 13, 0, 0, 0}, {12, 9, 16, 0, 0, 0},
+    {8, 6, 10, 0, 0, 0}, {10, 7, 13, 0, 0, 0}, {12, 9, 16, 0, 0, 0},
     {16, 10, 20, 0, 0, 0}, {24, 15, 25, 0, 0, 0}, {36, 25, 45, 0, 0, 0}};
 #define NFONT (sizeof(sys_fontlist)/sizeof(*sys_fontlist))
 
@@ -175,8 +178,6 @@ static void openit(const char *dirname, const char *filename)
         error("%s: can't open", filename);
 }
 
-#define NHOSTFONT 7
-
 /* this is called from the gui process.  The first argument is the cwd, and
 succeeding args give the widths and heights of known fonts.  We wait until 
 these are known to open files and send messages specified on the command line.
@@ -189,25 +190,33 @@ void glob_initfromgui(void *dummy, t_symbol *s, int argc, t_atom *argv)
 {
     char *cwd = atom_getsymbolarg(0, argc, argv)->s_name;
     t_namelist *nl;
-    unsigned int i, j;
-    if (argc != 2 + 3 * NHOSTFONT) bug("glob_initfromgui");
+    unsigned int i;
+    int j;
+    int nhostfont = (argc-2)/3;
+    sys_oldtclversion = atom_getfloatarg(1, argc, argv);
+    if (argc != 2 + 3 * nhostfont) bug("glob_initfromgui");
     for (i = 0; i < NFONT; i++)
     {
+        int best = 0;
         int wantheight = sys_fontlist[i].fi_maxheight;
-        for (j = 0; j < NHOSTFONT-1; j++)
+        int wantwidth = sys_fontlist[i].fi_maxwidth;
+        for (j = 1; j < nhostfont; j++)
         {
-            if (atom_getintarg(3 * (j + 1) + 3, argc, argv) > wantheight)
-                    break;
+            if (atom_getintarg(3 * j + 4, argc, argv) <= wantheight &&
+                atom_getintarg(3 * j + 3, argc, argv) <= wantwidth)
+                    best = j;
         }
-            /* j is now the "real" font index for the desired font index i. */
-        sys_fontlist[i].fi_hostfontsize = atom_getintarg(3 * j + 1, argc, argv);
-        sys_fontlist[i].fi_width = atom_getintarg(3 * j + 2, argc, argv);
-        sys_fontlist[i].fi_height = atom_getintarg(3 * j + 3, argc, argv);
+            /* best is now the host font index for the desired font index i. */
+        sys_fontlist[i].fi_hostfontsize =
+            atom_getintarg(3 * best + 2, argc, argv);
+        sys_fontlist[i].fi_width = atom_getintarg(3 * best + 3, argc, argv);
+        sys_fontlist[i].fi_height = atom_getintarg(3 * best + 4, argc, argv);
     }
 #if 0
     for (i = 0; i < 6; i++)
-        fprintf(stderr, "font %d %d %d %d %d\n",
+        fprintf(stderr, "font (%d %d %d) -> (%d %d %d)\n",
             sys_fontlist[i].fi_fontsize,
+            sys_fontlist[i].fi_maxwidth,
             sys_fontlist[i].fi_maxheight,
             sys_fontlist[i].fi_hostfontsize,
             sys_fontlist[i].fi_width,
@@ -215,7 +224,7 @@ void glob_initfromgui(void *dummy, t_symbol *s, int argc, t_atom *argv)
 #endif
         /* load dynamic libraries specified with "-lib" args */
     for  (nl = sys_externlist; nl; nl = nl->nl_next)
-        if (!sys_load_lib(cwd, nl->nl_string))
+        if (!sys_load_lib(0, nl->nl_string))
             post("%s: can't load library", nl->nl_string);
         /* open patches specifies with "-open" args */
     for  (nl = sys_openlist; nl; nl = nl->nl_next)
@@ -232,14 +241,23 @@ void glob_initfromgui(void *dummy, t_symbol *s, int argc, t_atom *argv)
     }
     namelist_free(sys_messagelist);
     sys_messagelist = 0;
-    sys_oldtclversion = atom_getfloatarg(1 + 3 * NHOSTFONT, argc, argv);
 }
 
 static void sys_afterargparse(void);
 
+static void pd_makeversion(void)
+{
+    char foo[100];
+    sprintf(foo,  "Pd version %d.%d-%d%s\n",PD_MAJOR_VERSION,
+        PD_MINOR_VERSION,PD_BUGFIX_VERSION,PD_TEST_VERSION);
+    pd_version = malloc(strlen(foo)+1);
+    strcpy(pd_version, foo);
+}
+
 /* this is called from main() in s_entry.c */
 int sys_main(int argc, char **argv)
 {
+    int i, noprefs;
     sys_externalschedlib = 0;
     sys_extraflags = 0;
 #ifdef PD_DEBUG
@@ -247,13 +265,19 @@ int sys_main(int argc, char **argv)
 #endif
     pd_init();                                  /* start the message system */
     sys_findprogdir(argv[0]);                   /* set sys_progname, guipath */
-    sys_loadpreferences();                      /* load default settings */
+    for (i = noprefs = 0; i < argc; i++)        /* prescan args for noprefs */
+        if (!strcmp(argv[i], "-noprefs"))
+            noprefs = 1;
+    if (!noprefs)
+        sys_loadpreferences();                  /* load default settings */
 #ifndef MSW
     sys_rcfile();                               /* parse the startup file */
 #endif
     if (sys_argparse(argc-1, argv+1))           /* parse cmd line */
         return (1);
     sys_afterargparse();                    /* post-argparse settings */
+        /* build version string from defines in m_pd.h */
+    pd_makeversion();
     if (sys_verbose || sys_version) fprintf(stderr, "%scompiled %s %s\n",
         pd_version, pd_compiletime, pd_compiledate);
     if (sys_version)    /* if we were just asked our version, exit here. */
@@ -369,6 +393,7 @@ static char *(usagemessage[]) = {
 "-guiport <n>     -- connect to pre-existing GUI over port <n>\n",
 "-guicmd \"cmd...\" -- start alternatve GUI program (e.g., remote via ssh)\n",
 "-send \"msg...\"   -- send a message at startup, after patches are loaded\n",
+"-noprefs         -- suppress loading preferences on startup\n",
 #ifdef UNISTD
 "-rt or -realtime -- use real-time priority\n",
 "-nrt             -- don't use real-time priority\n",
@@ -517,7 +542,7 @@ int sys_argparse(int argc, char **argv)
             argc -= 2;
             argv += 2;
         }
-        else if (!strcmp(*argv, "-inchannels"))
+        else if (!strcmp(*argv, "-inchannels") && (argc > 1))
         {
             sys_parsedevlist(&sys_nchin,
                 sys_chinlist, MAXAUDIOINDEV, argv[1]);
@@ -527,7 +552,7 @@ int sys_argparse(int argc, char **argv)
 
           argc -= 2; argv += 2;
         }
-        else if (!strcmp(*argv, "-outchannels"))
+        else if (!strcmp(*argv, "-outchannels") && (argc > 1))
         {
             sys_parsedevlist(&sys_nchout, sys_choutlist,
                 MAXAUDIOOUTDEV, argv[1]);
@@ -537,7 +562,7 @@ int sys_argparse(int argc, char **argv)
 
           argc -= 2; argv += 2;
         }
-        else if (!strcmp(*argv, "-channels"))
+        else if (!strcmp(*argv, "-channels") && (argc > 1))
         {
             sys_parsedevlist(&sys_nchin, sys_chinlist,MAXAUDIOINDEV,
                 argv[1]);
@@ -549,7 +574,7 @@ int sys_argparse(int argc, char **argv)
 
             argc -= 2; argv += 2;
         }
-        else if (!strcmp(*argv, "-soundbuf") || !strcmp(*argv, "-audiobuf"))
+        else if (!strcmp(*argv, "-soundbuf") || !strcmp(*argv, "-audiobuf") && (argc > 1))
         {
             sys_main_advance = atoi(argv[1]);
             argc -= 2; argv += 2;
@@ -559,7 +584,7 @@ int sys_argparse(int argc, char **argv)
             sys_setblocksize(atoi(argv[1]));
             argc -= 2; argv += 2;
         }
-        else if (!strcmp(*argv, "-sleepgrain"))
+        else if (!strcmp(*argv, "-sleepgrain") && (argc > 1))
         {
             sys_sleepgrain = 1000 * atoi(argv[1]);
             argc -= 2; argv += 2;
@@ -601,7 +626,7 @@ int sys_argparse(int argc, char **argv)
             sys_set_audio_api(API_ALSA);
             argc--; argv++;
         }
-        else if (!strcmp(*argv, "-alsaadd"))
+        else if (!strcmp(*argv, "-alsaadd") && (argc > 1))
         {
             if (argc > 1)
                 alsa_adddev(argv[1]);
@@ -664,7 +689,7 @@ int sys_argparse(int argc, char **argv)
                 goto usage;
             argc -= 2; argv += 2;
         }
-        else if (!strcmp(*argv, "-midioutdev"))
+        else if (!strcmp(*argv, "-midioutdev") && (argc > 1))
         {
             sys_parsedevlist(&sys_nmidiout, sys_midioutdevlist, MAXMIDIOUTDEV,
                 argv[1]);
@@ -672,7 +697,7 @@ int sys_argparse(int argc, char **argv)
                 goto usage;
             argc -= 2; argv += 2;
         }
-        else if (!strcmp(*argv, "-mididev"))
+        else if (!strcmp(*argv, "-mididev") && (argc > 1))
         {
             sys_parsedevlist(&sys_nmidiin, sys_midiindevlist, MAXMIDIINDEV,
                 argv[1]);
@@ -682,7 +707,7 @@ int sys_argparse(int argc, char **argv)
                 goto usage;
             argc -= 2; argv += 2;
         }
-        else if (!strcmp(*argv, "-path"))
+        else if (!strcmp(*argv, "-path") && (argc > 1))
         {
             sys_searchpath = namelist_append_files(sys_searchpath, argv[1]);
             argc -= 2; argv += 2;
@@ -721,10 +746,11 @@ int sys_argparse(int argc, char **argv)
                 /* tb: font name { */
         else if (!strcmp(*argv, "-typeface") && argc > 1)
         {
-                        strcpy(sys_font,*(argv+1));
-                        argc -= 2;
-                        argv += 2;
-                }
+            strncpy(sys_font,*(argv+1),sizeof(sys_font)-1);
+            sys_font[sizeof(sys_font)-1] = 0;
+            argc -= 2;
+            argv += 2;
+        }
                 /* } tb */
         else if (!strcmp(*argv, "-verbose"))
         {
@@ -775,7 +801,7 @@ int sys_argparse(int argc, char **argv)
         }
         else if (!strcmp(*argv, "-listdev"))
         {
-            sys_listdevs();
+            sys_listplease = 1;
             argc--; argv++;
         }
         else if (!strcmp(*argv, "-schedlib"))
@@ -822,7 +848,8 @@ int sys_argparse(int argc, char **argv)
                 goto usage;
             argc -= 2; argv += 2;
         }
-        else if (!strcmp(*argv, "-sounddev") || !strcmp(*argv, "-audiodev"))
+        else if ((!strcmp(*argv, "-sounddev") || !strcmp(*argv, "-audiodev"))
+                 && (argc > 1))
         {
             sys_parsedevlist(&sys_nsoundin, sys_soundindevlist,
                 MAXAUDIOINDEV, argv[1]);
@@ -832,6 +859,8 @@ int sys_argparse(int argc, char **argv)
                 goto usage;
             argc -= 2; argv += 2;
         }
+        else if (!strcmp(*argv, "-noprefs")) /* did this earlier */
+            argc--, argv++;
         else
         {
             unsigned int i;
@@ -892,6 +921,8 @@ static void sys_afterargparse(void)
         sys_midiindevlist[i]--;
     for (i = 0; i < sys_nmidiout; i++)
         sys_midioutdevlist[i]--;
+    if (sys_listplease)
+        sys_listdevs();
         
             /* get the current audio parameters.  These are set
             by the preferences mechanism (sys_loadpreferences()) or
@@ -944,8 +975,8 @@ static void sys_afterargparse(void)
         advance = sys_main_advance;
     if (sys_main_srate)
         rate = sys_main_srate;
-    sys_open_audio(naudioindev, audioindev, naudioindev, chindev,
-        naudiooutdev, audiooutdev, naudiooutdev, choutdev, rate, advance, 0);
+    sys_open_audio(naudioindev, audioindev, nchindev, chindev,
+        naudiooutdev, audiooutdev, nchoutdev, choutdev, rate, advance, 0);
     sys_open_midi(nmidiindev, midiindev, nmidioutdev, midioutdev, 0);
 }
 
